package org;

import java.net.*;
import java.io.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import org.util.xml.io.*;
import org.util.xml.element.*;

/**
 * 
 */
public class RealPlayer {

	private VirtualPlayer connected_virtual_player_;
	private int rcid_, vtid_;
	private String nickname_;
	
	public RealPlayer(String ip, int port, int timeout, TagElement tag) {
		init(ip, port, timeout, tag);
	}
	public RealPlayer(String ip, int port, int timeout, String nickname,int rcid, int vtid) {
		init(ip, port, timeout, nickname, rcid, vtid);
	}
	public String getID() {
		return rcid_+":"+vtid_;
	}
	public String getNickName() {
		return nickname_;
	}

	public VirtualPlayer getConnectedVirtualPlayer() {
		return connected_virtual_player_;
	}

	private XMLConfigManager config_;
	private DatagramPacket send_packet_;
	private DatagramPacket receive_packet_;
	private DatagramSocket socket_;

	public void init(String ip, int port, int timeout,TagElement tag) {
		String nickname = tag.getChildValue("nickname");
		int rcid = Integer.parseInt(tag.getChildValue("rc_id"));
		int vtid = Integer.parseInt(tag.getChildValue("vt_id"));
		init(ip, port, timeout, nickname, rcid, vtid);
	}
	public void init(String ip, int port, int timeout, String nickname, int rcid, int vtid) {
		try{
			initSocket(ip, port, timeout, nickname, rcid, vtid);
			TagElement tag = createFirstMessage();
			send(tag);
			receive();
			receive();
			send("<connect>true</connect>");

//			new Thread(new Runnable(){public void run(){try{
//				for(String message=receive(); message!=null; message=receive()) {
//					alert(message);
//					send(createKickMessage(0.1, 1));
//				}
//			}catch(Exception e){ e.printStackTrace(); }}}).start();

		}catch(Exception e){ e.printStackTrace(); }
	}
	public void initSocket(String ip, int port, int timeout, String nickname, int rcid, int vtid) throws Exception {
		try{
			nickname_ = nickname;
			rcid_ = rcid;
			vtid_ = vtid;
			InetSocketAddress address = new InetSocketAddress(ip, port);
			send_packet_ = new DatagramPacket(new byte[0], 0, address);
			byte[] receive_buf = new byte[1024*4];
			receive_packet_ = new DatagramPacket(receive_buf, receive_buf.length);
			socket_ = new DatagramSocket();
			socket_.connect(address);
			socket_.setSoTimeout(timeout);
		}catch(Exception e){
			e.printStackTrace();
		}
	}
	
	//130
	public TagElement createVerocitiesMessage(double right, double left) throws Exception {
		TagElement wheel_velocities = new TagElement("wheel_velocities");
		wheel_velocities.setChildren(
			new TagElement("right", String.valueOf(right)),
			new TagElement("left", String.valueOf(left))
		);
		TagElement command = new TagElement("command");
		command.setChildren(wheel_velocities);
		return command;
	}

	/*
	*/
	public TagElement createKickMessage(double angle, int force) throws Exception {
		TagElement kick = new TagElement("kick");
		kick.setChildren(
			new TagElement("angle", String.valueOf(angle)),
			new TagElement("force", String.valueOf(force))
		);
		TagElement command = new TagElement("command");
		command.setChildren(kick);
		return command;
	}

	public TagElement createFirstMessage() throws Exception {
		TagElement message = new TagElement("connect");
		message.setChildren(
			new TagElement("type", "Client"),
			new TagElement("protocol_version", "1.0"),
			new TagElement("nickname", nickname_),
			new TagElement("rc_id", String.valueOf(rcid_)),
			new TagElement("vt_id", String.valueOf(vtid_))
		);
		return message;
	}
/*
	public void send(String message) throws Exception {
		write(message);
		message = read();
		System.out.println("send: "+message);
	}
	public String receive() throws Exception {
		String message = read();
//		int len = message.split("\\|").length;
//		System.out.println("debug: "+len+"["+message+"]");
//		for(int i=0;i<len;i++)
//			write("<connect>true</connect>");
		write("<connect>true</connect>");
		System.out.println("receive: "+message);
		return message;
	}
*/	
	public void send(TagElement tag) throws Exception {
		send(tag.toString());
	}
	public void send(String message) throws Exception {
		byte[] write_buf = message.getBytes();
		send_packet_.setData(write_buf);
		socket_.send(send_packet_);
//		System.out.println("send: "+message);
	}
	public TagElement receiveTag() throws Exception {
		TagElement tag = XMLIO.read(new StringBuffer(receive()));
		return tag;
	}
	public String receive() throws Exception {
//		DatagramPacket rtmp = new DatagramPacket(receive_buf_, receive_buf_.length);
//		rtmp = packet_;
		socket_.receive(receive_packet_);
		String response = new String(receive_packet_.getData(), 0, receive_packet_.getLength());
//		System.out.println("receive: "+response);
		return response;
	}
	
	public static void alert(Object message) {
		System.out.println(message);
//		javax.swing.JOptionPane.showMessageDialog(null, message);
	}
}
