package org.agent;

import org.util.*;
import org.util.xml.io.*;
import org.util.xml.element.*;
import org.*;
import java.awt.*;
import javax.swing.*;
import java.awt.geom.*;
import java.awt.image.*;
import java.awt.event.*;

public class FieldPanel extends JComponent {

	private BufferedImage image_;
	private WorldData world_data_;
	private double center_x_;
	private double center_y_;
	private double zoom_x_ = -1.0;
	private double zoom_y_ = -1.0;
	private double pole_rad_view_width_;
	private double orientation_view_width_;
	private double agent_view_r_ = 4;
	private double dot_view_r_ = 2;
	private Stroke line_stroke_;
	private Stroke orientation_stroke_;
	private Graphics2D user_graphics_access_;
	private Graphics2D user_graphics_access_no_translated_;

	public FieldPanel() {
		setPreferredSize(new Dimension(400, 300));
		addComponentListener(new ComponentAdapter(){
			public void componentResized(ComponentEvent e) {
				createImage();
			}
		});
		addMouseWheelListener(new MouseWheelListener(){
			public void mouseWheelMoved(MouseWheelEvent e) {
				int r = e.getWheelRotation();
				if(r<0) {
					zoom_x_ *= 0.9;
					zoom_y_ *= 0.9;
				} else {
					zoom_x_ *= 1.1;
					zoom_y_ *= 1.1;
				}
				draw();
			}
		});
	}

	public void createImage() {
		int w = getWidth();
		int h = getHeight();
		if(image_==null || image_.getWidth()<w || image_.getHeight()<h)
			image_ = new BufferedImage(w, h, BufferedImage.TYPE_INT_ARGB);
		try{
			draw();
		}catch(Exception e){
			e.printStackTrace();
		}
	}
	
	public void update(WorldData world_data) {
		world_data_ = world_data;
		try{
			draw();
		}catch(Exception e){
			e.printStackTrace();
		}
	}
	
	public void draw()  {

		int w = getWidth();
		int h = getHeight();
		center_x_ = w/2;
		center_y_ = h/2;
		pole_rad_view_width_ = 3;
		orientation_view_width_ = 5;
		line_stroke_ = new BasicStroke((float)3);
		orientation_stroke_ = new BasicStroke((float)2);
		
		if(image_ == null) {
			new Thread(new Runnable(){public void run(){
				createImage();
			}}).start();
			return;
		}
		user_graphics_access_ = null;
		Graphics2D g2 = (Graphics2D)image_.getGraphics();
		Graphics2D g2_no_translated = (Graphics2D)g2.create();
//		g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
		g2.setColor(Color.green);
		g2.fillRect(0, 0, w, h);

		if(world_data_ == null) {
			g2.setFont(g2.getFont().deriveFont((float)30));
			g2.setColor(Color.red);
			g2.drawString("No data", 10, 40);
			return;
		}
		
		double tblength = world_data_.topRightCorner().sub(world_data_.bottomRightCorner()).dis();
		agent_view_r_ = 0.03*tblength;
		if(zoom_x_ == -1) {
			zoom_y_ = getHeight()/tblength;
			zoom_x_ = zoom_y_;
		}
				
		//g2.rotate(-Math.PI/2);

		g2.translate(center_x_, center_y_);
		g2.scale(zoom_x_, zoom_y_);

		g2.setColor(Color.white);
		g2.setStroke(line_stroke_);
		drawLine(g2, world_data_.bottomRightCorner(), world_data_.bottomLeftCorner());
		drawLine(g2, world_data_.bottomRightCorner(), world_data_.topRightCorner());
		drawLine(g2, world_data_.topLeftCorner(), world_data_.topRightCorner());
		drawLine(g2, world_data_.topLeftCorner(), world_data_.bottomLeftCorner());

		drawLine(g2, world_data_.topCenter(), world_data_.bottomCenter());
		
		drawLine(g2, world_data_.bottomRightPole(), world_data_.topRightSmallArea());
		drawLine(g2, world_data_.topRightSmallArea(), world_data_.bottomRightSmallArea());
		drawLine(g2, world_data_.bottomRightSmallArea(), world_data_.topRightPole());

		drawLine(g2, world_data_.bottomLeftPole(), world_data_.topLeftSmallArea());
		drawLine(g2, world_data_.topLeftSmallArea(), world_data_.bottomLeftSmallArea());
		drawLine(g2, world_data_.bottomLeftSmallArea(), world_data_.topLeftPole());
		
		drawDot(g2, world_data_.bottomLeftGoal(), dot_view_r_);
		drawDot(g2, world_data_.bottomRightGoal(), dot_view_r_);
		drawDot(g2, world_data_.topLeftGoal(), dot_view_r_);
		drawDot(g2, world_data_.topRightGoal(), dot_view_r_);
		
		g2_no_translated.setColor(Color.black);
		g2_no_translated.drawString(String.valueOf(world_data_.time()), 10, 10);


		g2.setStroke(orientation_stroke_);
		g2.setColor(Color.blue);
		AgentState[] team_mate_list = world_data_.teamMates();
		for(AgentState team_mate : team_mate_list)
			drawAgent(g2, team_mate);

		g2.setColor(Color.red);
		AgentState[] opponent_list = world_data_.opponents();
		for(AgentState opponent : opponent_list)
			drawAgent(g2, opponent);

		Vector ball = world_data_.ball();
		g2.setColor(Color.black);
		drawDot(g2, ball, 5);
		g2.setColor(Color.white);
		drawDot(g2, ball, 4);
//		drawString(g2_no_translated, "ball", ball);

		g2.setColor(Color.gray);
		g2.fill(new Arc2D.Double(-agent_view_r_, -agent_view_r_, agent_view_r_*2, agent_view_r_*2, 0, 360, Arc2D.CHORD));

		g2.setColor(Color.black);

		user_graphics_access_ = g2;
		user_graphics_access_no_translated_ = g2_no_translated;

		repaint();
	}
	
	/**
	 * you can set Color, Strock or RenderingHints for your draw.
	 */
	public Graphics2D getUserGraphicsAccess() throws Exception {
		if(user_graphics_access_ == null) throw new Exception("cannot write while drawing by system.");
		return user_graphics_access_;
	}
	
	
	
	/**
	 * repaint() have to call after this method.
	 */
	private void drawLine(Graphics2D g2, Vector v1, Vector v2) {
//		g2.draw(new Line2D.Double(v1.calcX(), v1.calcY(), v2.calcX(), v2.calcY()));
		g2.draw(new Line2D.Double(v1.calcY(), -v1.calcX(), v2.calcY(), -v2.calcX()));
	}
	private void drawDot(Graphics2D g2, Vector v, double r) {
//		g2.fill(new Arc2D.Double(v.calcX()-r, v.calcY()-r-1, r+r+1, r+r+1, 0, 360, Arc2D.CHORD));
		g2.fill(new Arc2D.Double(v.calcY()-r, -v.calcX()-r-1, r+r+1, r+r+1, 0, 360, Arc2D.CHORD));
	}
	private void drawString(Graphics2D g2, String message, Vector v) {
//		g2.drawString(message, (int)(v.calcX()*zoom_x_+center_x_+3), (int)(v.calcY()*zoom_y_+center_y_-3));
		g2.drawString(message, (int)(v.calcY()*zoom_x_+center_x_+3), (int)(-v.calcX()*zoom_y_+center_y_-3));
	}





	private void drawAgent(Graphics2D g2, AgentState agent) {
		if(agent==null) return ;
		org.util.Vector location = agent.getLocation();
		org.util.Vector orientation = agent.getOrientation();
		if(location == null) return ;
		double r = agent_view_r_;
		Color c = g2.getColor();
		g2.setColor(Color.black);
		drawDot(g2, location, r+1);
		g2.setColor(c);
		drawDot(g2, location, r);
		g2.setColor(Color.black);
		drawLine(g2, location, location.add(orientation.normalize(10)));
		g2.setColor(c);
	}

	
	public void drawLine(Vector v1, Vector v2) {
		try{
			Graphics2D g2 = getUserGraphicsAccess();
			drawLine(g2, v1, v2);
			repaint();
		}catch(Exception e){
			e.printStackTrace();
		}
	}
	public void drawDot(Vector v) {
		try{
			Graphics2D g2 = getUserGraphicsAccess();
			drawDot(g2, v, 3);
			repaint();
		}catch(Exception e){
			e.printStackTrace();
		}
	}
	public void drawString(String message, Vector v) {
		try{
			Graphics2D g2 = getUserGraphicsAccess();
			g2 = user_graphics_access_no_translated_;
			drawString(g2, message, v);
			repaint();
		}catch(Exception e){
			e.printStackTrace();
		}
	}

	public void paint(Graphics g) {
		int w = getWidth();
		int h = getHeight();
		if(image_ == null) {
			g.drawString("now loading...", 10, 10);
			new Thread(new Runnable(){public void run(){
				createImage();
			}}).start();
		} else
			g.drawImage(image_, 0, 0, this);
	}

	public static void alert(Object message) {
		System.out.println(message);
	}
}

/*
<WorldData>
   <time>5.25131</time>
   <agent_id>0</agent_id>
   <nickname>player1</nickname>
   <status>found</status>
   <max_agent>2</max_agent>
   <playMode>play on</playMode>
   <score>
      <yellow>0</yellow>
      <blue>0</blue>
   </score>
   <wheel_velocities>
      <right>0</right>
      <left>0</left>
   </wheel_velocities>
   <ball>
      <dist>117</dist>
      <angle>39</angle>
   </ball>
   <teamMate>
      <id>1</id>
      <nickname>player2</nickname>
      <status>found</status>
      <dist>89</dist>
      <angle>48</angle>
      <orientation>107</orientation>
   </teamMate>
   <opponent>
      <id>5</id>
      <nickname>player7</nickname>
      <status>found</status>
      <dist>167</dist>
      <angle>46</angle>
      <orientation>161</orientation>
   </opponent>
   <flag>
      <id>top_left_pole</id>
      <dist>152</dist>
      <angle>-59</angle>
   </flag>
   <flag>
      <id>bottom_left_pole</id>
      <dist>116</dist>
      <angle>-83</angle>
   </flag>
   <flag>
      <id>top_right_pole</id>
      <dist>310</dist>
      <angle>57</angle>
   </flag>
   <flag>
      <id>bottom_right_pole</id>
      <dist>294</dist>
      <angle>69</angle>
   </flag>
   <flag>
      <id>top_left_corner</id>
      <dist>236</dist>
      <angle>-40</angle>
   </flag>
   <flag>
      <id>bottom_left_corner</id>
      <dist>126</dist>
      <angle>-133</angle>
   </flag>
   <flag>
      <id>top_right_corner</id>
      <dist>359</dist>
      <angle>41</angle>
   </flag>
   <flag>
      <id>bottom_right_corner</id>
      <dist>298</dist>
      <angle>89</angle>
   </flag>
   <flag>
      <id>top_left_goal</id>
      <dist>148</dist>
      <angle>-8</angle>
   </flag>
   <flag>
      <id>bottom_left_goal</id>
      <dist>14</dist>
      <angle>85</angle>
   </flag>
   <flag>
      <id>top_right_goal</id>
      <dist>224</dist>
      <angle>36</angle>
   </flag>
   <flag>
      <id>bottom_right_goal</id>
      <dist>169</dist>
      <angle>78</angle>
   </flag>
   <flag>
      <id>top_left_small_area</id>
      <dist>83</dist>
      <angle>-74</angle>
   </flag>
   <flag>
      <id>bottom_left_small_area</id>
      <dist>128</dist>
      <angle>-48</angle>
   </flag>
   <flag>
      <id>top_right_small_area</id>
      <dist>259</dist>
      <angle>68</angle>
   </flag>
   <flag>
      <id>bottom_right_small_area</id>
      <dist>277</dist>
      <angle>55</angle>
   </flag>
   <flag>
      <id>middle_center</id>
      <dist>117</dist>
      <angle>39</angle>
   </flag>
   <flag>
      <id>top_center</id>
      <dist>229</dist>
      <angle>11</angle>
   </flag>
   <flag>
      <id>bottom_center</id>
      <dist>112</dist>
      <angle>112</angle>
   </flag>
</WorldData>
*/
