package org.agent;

import org.util.xml.io.*;
import org.util.xml.element.*;
import org.*;
import org.util.*;

import javax.swing.*;

public class WorldData {

	private double time_;
	private int agent_id_;
	private String nickname_;
	private String status_;
	private int max_agent_;
	private String play_mode_;
	private int yellow_score_;
	private int blue_score_;
	private Vector ball_;
	private AgentState[] team_mate_list_;
	private AgentState[] opponent_list_;
	private Vector top_left_pole_;
	private Vector bottom_left_pole_;
	private Vector top_right_pole_;
	private Vector bottom_right_pole_;
	private Vector top_left_corner_;
	private Vector bottom_left_corner_;
	private Vector top_right_corner_;
	private Vector bottom_right_corner_;
	private Vector top_left_goal_;
	private Vector bottom_left_goal_;
	private Vector top_right_goal_;
	private Vector bottom_right_goal_;
	private Vector top_left_small_area_;
	private Vector bottom_left_small_area_;
	private Vector top_right_small_area_;
	private Vector bottom_right_small_area_;
	private Vector middle_center_;
	private Vector top_center_;
	private Vector bottom_center_;
	
	public WorldData() {
		
	}
	
	
	public double time() {
		return time_;
	}
	public int agentID() {
		return agent_id_;
	}
	public String nickname() {
		return nickname_;
	}
	public String status() {
		return status_;
	}
	public int maxAgent() {
		return max_agent_;
	}
	public String playMode() {
		return play_mode_;
	}
	public int yellowScore() {
		return yellow_score_;
	}
	public int blueScore() {
		return blue_score_;
	}
	public Vector ball() {
		return ball_;
	}
	public AgentState[] teamMates() {
		return team_mate_list_;
	}
	public AgentState[] opponents() {
		return opponent_list_;
	}
	/**
	 * 1,2,3...max
	 * except me.
	 */
	public AgentState teamMate(int id) {
		return team_mate_list_[id-1];
	}
	/**
	 * 1,2,3...max
	 */
	public AgentState opponent(int id) {
		return opponent_list_[id-1];
	}



	public Vector topLeftPole() {
		return top_left_pole_;
	}
	public Vector bottomLeftPole() {
		return bottom_left_pole_;
	}
	public Vector topRightPole() {
		return top_right_pole_;
	}
	public Vector bottomRightPole() {
		return bottom_right_pole_;
	}


	public Vector topLeftCorner() {
		return top_left_corner_;
	}
	public Vector bottomLeftCorner() {
		return bottom_left_corner_;
	}
	public Vector topRightCorner() {
		return top_right_corner_;
	}
	public Vector bottomRightCorner() {
		return bottom_right_corner_;
	}



	public Vector topLeftGoal() {
		return top_left_goal_;
	}
	public Vector bottomLeftGoal() {
		return bottom_left_goal_;
	}
	public Vector topRightGoal() {
		return top_right_goal_;
	}
	public Vector bottomRightGoal() {
		return bottom_right_goal_;
	}



	public Vector topLeftSmallArea() {
		return top_left_small_area_;
	}
	public Vector bottomLeftSmallArea() {
		return bottom_left_small_area_;
	}
	public Vector topRightSmallArea() {
		return top_right_small_area_;
	}
	public Vector bottomRightSmallArea() {
		return bottom_right_small_area_;
	}


	public Vector middleCenter() {
		return middle_center_;
	}
	public Vector topCenter() {
		return top_center_;
	}
	public Vector bottomCenter() {
		return bottom_center_;
	}
	
	
	public Vector me() {
		return new Vector(0, 0);
	}


	public void update(TagElement world_data) throws Exception {
		time_ = getDouble(world_data, "time");
		agent_id_ = getInt(world_data, "agent_id");
		nickname_ = world_data.getChildValue("nickname");
		status_ = world_data.getChildValue("status");
		max_agent_ = getInt(world_data, "max_agent");
		if(team_mate_list_==null) {
			team_mate_list_ = new AgentState[Math.max(max_agent_/2-1, 0)];
			opponent_list_ = new AgentState[max_agent_/2];
		}
//		System.out.println();
		play_mode_ = world_data.getChildValue("play");
		TagElement score = world_data.getTagChild("score");
		yellow_score_ = getInt(score, "yellow");
		blue_score_ = getInt(score, "blue");
//		TagElement wheel_velocities = world_data.getTagChild("wheel_velocities");
//		left_wheel_velocity_ = getDouble(wheel_velocities, "left");
//		right_wheel_velocity_ = getDouble(wheel_velocities, "right");
		ball_ = getVector(world_data.getTagChild("ball"));
		TagElement[] team_mate_tag_list_ = world_data.getTagChildren("teamMate");
		for(int i=0; i<Math.min(team_mate_list_.length, team_mate_tag_list_.length); i++)
			try{
				team_mate_list_[i] = getAgentState(team_mate_tag_list_[i]);
			}catch(Exception e){e.printStackTrace();}

		TagElement[] opponent_tag_list_ = world_data.getTagChildren("opponent");
		for(int i=0; i<Math.min(opponent_list_.length, opponent_tag_list_.length); i++)
			try{
				opponent_list_[i] = getAgentState(opponent_tag_list_[i]);
			}catch(Exception e){e.printStackTrace();}
		
		TagElement[] flag_tag_list_ = world_data.getTagChildren("flag");
		top_left_pole_ = getVectorByID(flag_tag_list_, "top_left_pole");
		bottom_left_pole_ = getVectorByID(flag_tag_list_, "bottom_left_pole");
		top_right_pole_ = getVectorByID(flag_tag_list_, "top_right_pole");
		bottom_right_pole_ = getVectorByID(flag_tag_list_, "bottom_right_pole");
		
		top_left_corner_ = getVectorByID(flag_tag_list_, "top_left_corner");
		bottom_left_corner_ = getVectorByID(flag_tag_list_, "bottom_left_corner");
		top_right_corner_ = getVectorByID(flag_tag_list_, "top_right_corner");
		bottom_right_corner_ = getVectorByID(flag_tag_list_, "bottom_right_corner");

		top_left_goal_ = getVectorByID(flag_tag_list_, "top_left_goal");
		bottom_left_goal_ = getVectorByID(flag_tag_list_, "bottom_left_goal");
		top_right_goal_ = getVectorByID(flag_tag_list_, "top_right_goal");
		bottom_right_goal_ = getVectorByID(flag_tag_list_, "bottom_right_goal");

		top_left_small_area_ = getVectorByID(flag_tag_list_, "top_left_small_area");
		bottom_left_small_area_ = getVectorByID(flag_tag_list_, "bottom_left_small_area");
		top_right_small_area_ = getVectorByID(flag_tag_list_, "top_right_small_area");
		bottom_right_small_area_ = getVectorByID(flag_tag_list_, "bottom_right_small_area");
		
		middle_center_ = getVectorByID(flag_tag_list_, "middle_center");
		top_center_ = getVectorByID(flag_tag_list_, "top_center");
		bottom_center_ = getVectorByID(flag_tag_list_, "bottom_center");
	}

	public static int getInt(TagElement tag, String key) throws Exception {
		return Integer.parseInt(tag.getChildValue(key));
	}
	public static double getDouble(TagElement tag, String key) throws Exception {
		return Double.parseDouble(tag.getChildValue(key));
	}
	public static Vector getVectorByID(TagElement[] tags, String id) throws Exception {
		for(int i=0; i<tags.length; i++) {
			if(tags[i].getChildValue("id").equals(id))
				return getVector(tags[i]);
		}
		return null;
	}
	public static Vector getVector(TagElement tag) throws Exception {
		int angle = getInt(tag, "angle");
		double distance = getDouble(tag, "dist");
		return new Vector(angle, distance);
	}
	public static AgentState getAgentStateByID(TagElement[] tags, String id) throws Exception {
		for(int i=0; i<tags.length; i++) {
			if(tags[i].getChildValue("id").equals(id))
				return getAgentState(tags[i]);
		}
		return null;
	}
	
	public static AgentState getAgentState(TagElement tag) throws Exception {
		int id = getInt(tag, "id");
		String nickname = tag.getChildValue("nickname");
		int distance = (int)(getDouble(tag, "dist"));
		int angle = (int)roundDeg(getDouble(tag, "angle"));
		double orientation = getDouble(tag, "orientation");
		return new AgentState(id, nickname, new Vector(angle, distance), new Vector(orientation, 1.0));
	}
	
	public static double roundDeg(double deg) {
		while(deg>180) deg -= 360;
		while(deg<-180) deg += 360;
		return deg;
	}
}


/*
<WorldData>
   <time>5.25131</time>
   <agent_id>0</agent_id>
   <nickname>player1</nickname>
   <status>found</status>
   <max_agent>2</max_agent>
   <playMode>play on</playMode>
   <score>
      <yellow>0</yellow>
      <blue>0</blue>
   </score>
   <wheel_velocities>
      <right>0</right>
      <left>0</left>
   </wheel_velocities>
   <ball>
      <dist>117</dist>
      <angle>39</angle>
   </ball>
   <teamMate>
      <id>1</id>
      <nickname>player2</nickname>
      <status>found</status>
      <dist>89</dist>
      <angle>48</angle>
      <orientation>107</orientation>
   </teamMate>
   <opponent>
      <id>5</id>
      <nickname>player7</nickname>
      <status>found</status>
      <dist>167</dist>
      <angle>46</angle>
      <orientation>161</orientation>
   </opponent>
   <flag>
      <id>top_left_pole</id>
      <dist>152</dist>
      <angle>-59</angle>
   </flag>
   <flag>
      <id>bottom_left_pole</id>
      <dist>116</dist>
      <angle>-83</angle>
   </flag>
   <flag>
      <id>top_right_pole</id>
      <dist>310</dist>
      <angle>57</angle>
   </flag>
   <flag>
      <id>bottom_right_pole</id>
      <dist>294</dist>
      <angle>69</angle>
   </flag>
   <flag>
      <id>top_left_corner</id>
      <dist>236</dist>
      <angle>-40</angle>
   </flag>
   <flag>
      <id>bottom_left_corner</id>
      <dist>126</dist>
      <angle>-133</angle>
   </flag>
   <flag>
      <id>top_right_corner</id>
      <dist>359</dist>
      <angle>41</angle>
   </flag>
   <flag>
      <id>bottom_right_corner</id>
      <dist>298</dist>
      <angle>89</angle>
   </flag>
   <flag>
      <id>top_left_goal</id>
      <dist>148</dist>
      <angle>-8</angle>
   </flag>
   <flag>
      <id>bottom_left_goal</id>
      <dist>14</dist>
      <angle>85</angle>
   </flag>
   <flag>
      <id>top_right_goal</id>
      <dist>224</dist>
      <angle>36</angle>
   </flag>
   <flag>
      <id>bottom_right_goal</id>
      <dist>169</dist>
      <angle>78</angle>
   </flag>
   <flag>
      <id>top_left_small_area</id>
      <dist>83</dist>
      <angle>-74</angle>
   </flag>
   <flag>
      <id>bottom_left_small_area</id>
      <dist>128</dist>
      <angle>-48</angle>
   </flag>
   <flag>
      <id>top_right_small_area</id>
      <dist>259</dist>
      <angle>68</angle>
   </flag>
   <flag>
      <id>bottom_right_small_area</id>
      <dist>277</dist>
      <angle>55</angle>
   </flag>
   <flag>
      <id>middle_center</id>
      <dist>117</dist>
      <angle>39</angle>
   </flag>
   <flag>
      <id>top_center</id>
      <dist>229</dist>
      <angle>11</angle>
   </flag>
   <flag>
      <id>bottom_center</id>
      <dist>112</dist>
      <angle>112</angle>
   </flag>
</WorldData>
*/
