package org.server;

import java.net.*;
import java.io.*;
import java.util.*;
import java.awt.geom.*;

import org.realobject.*;
import org.server.*;

import org.util.xml.io.*;
import org.util.xml.element.*;
import org.server.motion.*;
/**
 *
 */
public class DummyServer {

	private static String config_file_name_ = "data/config-DummyServer.xml";

	private ArrayList<RCRSVirtualSoccerEcoBe> ecobe_list_ = new ArrayList<RCRSVirtualSoccerEcoBe>();
	private ArrayList<Mark> mark_list_ = new ArrayList<Mark>();
	private ArrayList<RCRSVirtualSoccerEcoBe> blue_team_member_list_ = new ArrayList<RCRSVirtualSoccerEcoBe>();
	private ArrayList<RCRSVirtualSoccerEcoBe> yellow_team_member_list_ = new ArrayList<RCRSVirtualSoccerEcoBe>();
	private ArrayList<DummyServerListener> dslistener_list_ = new ArrayList<DummyServerListener>();
	private Ball ball_;
//	private Point2D.Double ball_location_;
//	private double ball_radius_ = 7;
	private MotionSimulator motion_simulator_;
	
	private HashMap<String, Point2D.Double> flag_location_map_ = new HashMap<String, Point2D.Double>();
	private Rectangle2D flag_area_;
	private double time_ =0;

	public static void main(String[] args) throws Exception {

		if(args.length==1)
			config_file_name_ = args[0];

		System.out.println("Now starting DummyServer.");
		System.out.println("config file: " + config_file_name_);
		XMLConfigManager config = new XMLConfigManager(config_file_name_);
		DummyServer server = new DummyServer(config);

		new DummyServerGUI(server);

		server.start();
		
	}
	
	public void setMotionSimulator(MotionSimulator motion_simulator) {
		motion_simulator_ = motion_simulator;
		if(motion_simulator_!=null)
			motion_simulator_.init(flag_location_map_, ecobe_list_, ball_);
	}
	public MotionSimulator getMotionSimulator() {
		return motion_simulator_;
	}

	public ArrayList<RCRSVirtualSoccerEcoBe> getEcoBeList() {
		return ecobe_list_;
	}
	public ArrayList<RCRSVirtualSoccerEcoBe> getBlueTeamMemberList() {
		return blue_team_member_list_;
	}
	public ArrayList<RCRSVirtualSoccerEcoBe> getYellowTeamMemberList() {
		return yellow_team_member_list_;
	}
	public HashMap<String, Point2D.Double> getFlags() {
		return flag_location_map_;
	}
	public Rectangle2D getArea() {
		return flag_area_;
	}

	public DummyServer(XMLConfigManager config) throws Exception {
		init(config);
	}

	public Point2D.Double getBallLocation() {
		return ball_.getLocation();
	}

	public double getBallRadius() {
		return ball_.getRadius();
	}

	private XMLConfigManager config_;
	private DatagramPacket blue_packet_;
	private DatagramSocket blue_socket_;
	private byte[] blue_receive_buf_;
	private DatagramPacket yellow_packet_;
	private DatagramSocket yellow_socket_;
	private byte[] yellow_receive_buf_;

	public void init(XMLConfigManager config) throws Exception {
		config_ = config;
		flag_location_map_.put("top_left_pole", new Point2D.Double(0, 200));
		flag_location_map_.put("bottom_left_pole", new Point2D.Double(0, 300));
		flag_location_map_.put("bottom_right_pole", new Point2D.Double(1000, 200));
		flag_location_map_.put("top_right_pole", new Point2D.Double(1000, 300));

		flag_location_map_.put("top_left_corner", new Point2D.Double(0, 0));
		flag_location_map_.put("bottom_left_corner", new Point2D.Double(0, 500));
		flag_location_map_.put("top_right_corner", new Point2D.Double(1000, 0));
		flag_location_map_.put("bottom_right_corner", new Point2D.Double(1000, 500));
		
		flag_location_map_.put("top_left_goal", new Point2D.Double(00, 200));
		flag_location_map_.put("bottom_left_goal", new Point2D.Double(00, 300));
		flag_location_map_.put("top_right_goal", new Point2D.Double(1000, 200));
		flag_location_map_.put("bottom_right_goal", new Point2D.Double(1000, 300));

		flag_location_map_.put("bottom_left_small_area", new Point2D.Double(70, 200));
		flag_location_map_.put("top_left_small_area", new Point2D.Double(70, 300));
		flag_location_map_.put("top_right_small_area", new Point2D.Double(930, 200));
		flag_location_map_.put("bottom_right_small_area", new Point2D.Double(930, 300));

		flag_location_map_.put("middle_center", new Point2D.Double(500, 250));
		flag_location_map_.put("top_center", new Point2D.Double(500, 0));
		flag_location_map_.put("bottom_center", new Point2D.Double(500, 500));
		ball_ = new Ball(new Point2D.Double(100, 100), 15, 10);
		
		for(Iterator<Point2D.Double> it = flag_location_map_.values().iterator(); it.hasNext();) {
			Point2D.Double p = it.next();
			if(flag_area_==null)
				flag_area_ = new Rectangle2D.Double(p.getX(), p.getY(), 0, 0);
			else
				flag_area_.add(p);
		}
		
	}

	public void addDummyServerListener(DummyServerListener listener) {
		dslistener_list_.add(listener);
	}

	public void start() throws Exception {
		init();
		
		new Thread(new Runnable(){public void run(){
			startProcess();
		}}).start();

		// blue team packet receive thread
		new Thread(new Runnable(){public void run(){try{
			while(true) {
				blue_packet_.setData(blue_receive_buf_);
				blue_socket_.receive(blue_packet_);
				InetSocketAddress address = (InetSocketAddress)blue_packet_.getSocketAddress();
				RCRSVirtualSoccerEcoBe ecobe = findEcoBe(address);
				if(ecobe != null) {
					ecobe.update(blue_packet_);
				} else {
					try{
						tryToCreateNewBlueConnection(blue_packet_);
					}catch(Exception e) {
						e.printStackTrace();
					}
				}
			}
		}catch(Exception e){e.printStackTrace();}}}).start();

		// yellow team packet receive thread
		new Thread(new Runnable(){public void run(){try{
			while(true) {
				yellow_packet_.setData(yellow_receive_buf_);
				yellow_socket_.receive(yellow_packet_);
				InetSocketAddress address = (InetSocketAddress)yellow_packet_.getSocketAddress();
				RCRSVirtualSoccerEcoBe ecobe = findEcoBe(address);
				if(ecobe != null) {
					ecobe.update(yellow_packet_);
				} else {
					try{
						tryToCreateNewYellowConnection(yellow_packet_);
					}catch(Exception e) {
						e.printStackTrace();
					}
				}
			}
		}catch(Exception e){e.printStackTrace();}}}).start();

//		new Thread(new Runnable(){public void run(){
//		}}).start();		

	}


	
int step_length_ = 1000/15; //[ms]
int counter = 0;
int skip_counter_=0;
	public void startProcess() {
/*
String sampledata = "<WorldData><time>5.25131</time><agent_id>0</agent_id><nickname>player1</nickname><status>found</status><max_agent>2</max_agent><playMode>play on</playMode><score><yellow>0</yellow><blue>0</blue></score><wheel_velocities><right>0</right><left>0</left></wheel_velocities><ball><dist>117</dist><angle>39</angle></ball><teamMate><id>1</id><nickname>player2</nickname><status>found</status><dist>89</dist><angle>48</angle><orientation>107</orientation></teamMate><teamMate><id>2</id><nickname>player3</nickname><status>found</status><dist>79</dist><angle>-21</angle><orientation>103</orientation></teamMate><teamMate><id>3</id><nickname>player4</nickname><status>found</status><dist>74</dist><angle>-57</angle><orientation>273</orientation></teamMate><teamMate><id>4</id><nickname>player0</nickname><status>found</status><dist>70</dist><angle>66</angle><orientation>258</orientation></teamMate><opponent><id>5</id><nickname>player7</nickname><status>found</status><dist>167</dist><angle>46</angle><orientation>161</orientation></opponent><opponent><id>6</id><nickname>player9</nickname><status>found</status><dist>56</dist><angle>3</angle><orientation>249</orientation></opponent><opponent><id>7</id><nickname>player5</nickname><status>found</status><dist>287</dist><angle>58</angle><orientation>357</orientation></opponent><opponent><id>8</id><nickname>player8</nickname><status>found</status><dist>53</dist><angle>0</angle><orientation>328</orientation></opponent><opponent><id>9</id><nickname>player6</nickname><status>found</status><dist>128</dist><angle>-34</angle><orientation>6</orientation></opponent><flag><id>top_left_pole</id><dist>152</dist><angle>-59</angle></flag><flag><id>bottom_left_pole</id><dist>116</dist><angle>-83</angle></flag><flag><id>top_right_pole</id><dist>310</dist><angle>57</angle></flag><flag><id>bottom_right_pole</id><dist>294</dist><angle>69</angle></flag><flag><id>top_left_corner</id><dist>236</dist><angle>-40</angle></flag><flag><id>bottom_left_corner</id><dist>126</dist><angle>-133</angle></flag><flag><id>top_right_corner</id><dist>359</dist><angle>41</angle></flag><flag><id>bottom_right_corner</id><dist>298</dist><angle>89</angle></flag><flag><id>top_left_goal</id><dist>148</dist><angle>-8</angle></flag><flag><id>bottom_left_goal</id><dist>14</dist><angle>85</angle></flag><flag><id>top_right_goal</id><dist>224</dist><angle>36</angle></flag><flag><id>bottom_right_goal</id><dist>169</dist><angle>78</angle></flag><flag><id>top_left_small_area</id><dist>83</dist><angle>-74</angle></flag><flag><id>bottom_left_small_area</id><dist>128</dist><angle>-48</angle></flag><flag><id>top_right_small_area</id><dist>259</dist><angle>68</angle></flag><flag><id>bottom_right_small_area</id><dist>277</dist><angle>55</angle></flag><flag><id>middle_center</id><dist>117</dist><angle>39</angle></flag><flag><id>top_center</id><dist>229</dist><angle>11</angle></flag><flag><id>bottom_center</id><dist>112</dist><angle>112</angle></flag></WorldData>";
*/
//double sin = Math.sin((double)counter*0.01);
//double cos = Math.sin((double)counter*0.01);

		while(true) {
int rad = (counter+=5)%360-180;
		long last_time = System.currentTimeMillis();

final String sampledata2 = "<WorldData><time>"+time_+"</time><agent_id>0</agent_id><nickname>player1</nickname><status>found</status><max_agent>5</max_agent><playMode>play on</playMode><score><yellow>0</yellow><blue>0</blue></score><wheel_velocities><right>0</right><left>0</left></wheel_velocities><ball><dist>117</dist><angle>"+rad+"</angle></ball><teamMate><id>1</id><nickname>player2</nickname><status>found</status><dist>89</dist><angle>48</angle><orientation>107</orientation></teamMate><teamMate><id>2</id><nickname>player3</nickname><status>found</status><dist>79</dist><angle>-21</angle><orientation>103</orientation></teamMate><teamMate><id>3</id><nickname>player4</nickname><status>found</status><dist>74</dist><angle>-57</angle><orientation>273</orientation></teamMate><teamMate><id>4</id><nickname>player0</nickname><status>found</status><dist>70</dist><angle>66</angle><orientation>258</orientation></teamMate><opponent><id>5</id><nickname>player7</nickname><status>found</status><dist>167</dist><angle>46</angle><orientation>161</orientation></opponent><opponent><id>6</id><nickname>player9</nickname><status>found</status><dist>56</dist><angle>3</angle><orientation>249</orientation></opponent><opponent><id>7</id><nickname>player5</nickname><status>found</status><dist>287</dist><angle>58</angle><orientation>357</orientation></opponent><opponent><id>8</id><nickname>player8</nickname><status>found</status><dist>53</dist><angle>0</angle><orientation>328</orientation></opponent><opponent><id>9</id><nickname>player6</nickname><status>found</status><dist>128</dist><angle>-34</angle><orientation>6</orientation></opponent><flag><id>top_left_pole</id><dist>152</dist><angle>-59</angle></flag><flag><id>bottom_left_pole</id><dist>116</dist><angle>-83</angle></flag><flag><id>top_right_pole</id><dist>310</dist><angle>57</angle></flag><flag><id>bottom_right_pole</id><dist>294</dist><angle>69</angle></flag><flag><id>top_left_corner</id><dist>236</dist><angle>-40</angle></flag><flag><id>bottom_left_corner</id><dist>126</dist><angle>-133</angle></flag><flag><id>top_right_corner</id><dist>359</dist><angle>41</angle></flag><flag><id>bottom_right_corner</id><dist>298</dist><angle>89</angle></flag><flag><id>top_left_goal</id><dist>148</dist><angle>-8</angle></flag><flag><id>bottom_left_goal</id><dist>14</dist><angle>85</angle></flag><flag><id>top_right_goal</id><dist>224</dist><angle>36</angle></flag><flag><id>bottom_right_goal</id><dist>169</dist><angle>78</angle></flag><flag><id>top_left_small_area</id><dist>83</dist><angle>-74</angle></flag><flag><id>bottom_left_small_area</id><dist>128</dist><angle>-48</angle></flag><flag><id>top_right_small_area</id><dist>259</dist><angle>68</angle></flag><flag><id>bottom_right_small_area</id><dist>277</dist><angle>55</angle></flag><flag><id>middle_center</id><dist>117</dist><angle>39</angle></flag><flag><id>top_center</id><dist>229</dist><angle>11</angle></flag><flag><id>bottom_center</id><dist>112</dist><angle>112</angle></flag></WorldData>";

//System.out.println(sampledata2);

//log("send message: ");
			for(int i=0;i<blue_team_member_list_.size();i++) {
				RCRSVirtualSoccerEcoBe ecobe = blue_team_member_list_.get(i);
				String sampledata = createMessage(ecobe);
				try{
					if(ecobe.isConnected()) {
						blue_socket_.send(ecobe.createPacket(sampledata));
					}else{
//						System.out.println("skip: "+skip_counter_++);
					}
				}catch(Exception e) {
					e.printStackTrace();
				}
			}
			for(int i=0;i<yellow_team_member_list_.size();i++) {
				RCRSVirtualSoccerEcoBe ecobe = yellow_team_member_list_.get(i);
				String sampledata = createMessage(ecobe);
				try{
					if(ecobe.isConnected()) {
						yellow_socket_.send(ecobe.createPacket(sampledata));
					}else{
//						System.out.println("skip: "+skip_counter_++);
					}
				}catch(Exception e) {
					e.printStackTrace();
				}
			}

			if(motion_simulator_!=null)
				motion_simulator_.simulateStep(1);

			for(int j=0;j<dslistener_list_.size();j++)
				try{
					dslistener_list_.get(j).update();
				}catch(Exception e){e.printStackTrace();}

			long time = System.currentTimeMillis();
			int wait = step_length_ - (int)(time-last_time);
//			System.out.println("wait: "+wait);
			if(wait<0)
				System.out.println("cannot keep step length: "+step_length_+"<"+(last_time-time));
			else
				try{Thread.sleep(wait);}catch(Exception e) {}
			
			time_ += ((double)step_length_)/1000;
		}
	}
/*
	public void createClient(DatagramPacket packet) throws Exception {
		
		InetSocketAddress address = (InetSocketAddress)packet.getSocketAddress();
		for(int i=0;i<dslistener_list_.size();i++)
			dslistener_list_.get(i).connectionDetected((address));
		String message = new String(packet.getData(), 0, packet.getLength());
		TagElement tag = XMLIO.read(new StringBuffer(message));
		if(!tag.getChildValue("type").equals("Client"))
			throw new Exception("unsupported type: "+tag.getChildValue("type"));
		if(!tag.getChildValue("protocol_version").equals("1.0"))
			throw new Exception("unsupported protocol_version: "+tag.getChildValue("protocol_version"));
		String nickname = tag.getChildValue("nickname");
		int rc_id = Integer.parseInt(tag.getChildValue("rc_id"));
		int vt_id = Integer.parseInt(tag.getChildValue("vt_id"));

		EcoBe ecobe = findEcoBe(rc_id);
		Mark mark = findMark(vt_id);

		send(new_client.createPacket("<connect>true</connect>"));
		
		SoccerClient new_client = new SoccerClient(address, ecobe);
		client_list_.add(new_client);
		
		send(new_client.createPacket("<connect><type>Client</type><protocol_version>1.0</protocol_version><movement_mode>wheel_velocities</movement_mode></connect>"));
		
		for(int i=0;i<dslistener_list_.size();i++)
			dslistener_list_.get(i).failedToConnect(address, "???");
		
log("Connected new client: "+new_client);
	}
*/	
	public void tryToCreateNewYellowConnection(DatagramPacket packet) throws Exception {
		InetSocketAddress address = (InetSocketAddress)packet.getSocketAddress();
		RCRSVirtualSoccerEcoBe ecobe = null;
		for(int i=0;i<yellow_team_member_list_.size()&&ecobe==null;i++) {
			if(!yellow_team_member_list_.get(i).isConnected())
				ecobe = yellow_team_member_list_.get(i);
		}
		if(ecobe == null) throw new Exception("yellow team is already full");
		ecobe.setClient(address);
		ecobe.update(packet);
		yellow_socket_.send(ecobe.createPacket("<connect>true</connect>"));
		yellow_socket_.send(ecobe.createPacket("<connect><type>Client</type><protocol_version>1.0</protocol_version><movement_mode>wheel_velocities</movement_mode></connect>"));
		log("Connected new client(yellow): "+ecobe);
	}

	public void tryToCreateNewBlueConnection(DatagramPacket packet) throws Exception {
		InetSocketAddress address = (InetSocketAddress)packet.getSocketAddress();
		RCRSVirtualSoccerEcoBe ecobe = null;
		for(int i=0;i<blue_team_member_list_.size()&&ecobe==null;i++) {
			if(!blue_team_member_list_.get(i).isConnected())
				ecobe = blue_team_member_list_.get(i);
		}
		if(ecobe == null) throw new Exception("blue team is already full");
		ecobe.setClient(address);
		ecobe.update(packet);
		blue_socket_.send(ecobe.createPacket("<connect>true</connect>"));
		blue_socket_.send(ecobe.createPacket("<connect><type>Client</type><protocol_version>1.0</protocol_version><movement_mode>wheel_velocities</movement_mode></connect>"));
		log("Connected new client(blue): "+ecobe);
	}
/*
	public void tryToCreateNewConnection(DatagramPacket packet) throws Exception {

		InetSocketAddress address = (InetSocketAddress)packet.getSocketAddress();
		int port = address.getPort();
		
//		System.out.println(port);
		for(int i=0;i<dslistener_list_.size();i++)
			dslistener_list_.get(i).connectionDetected((address));
		String message = new String(packet.getData(), 0, packet.getLength());
		TagElement tag = XMLIO.read(new StringBuffer(message));
		if(!tag.getChildValue("type").equals("Client"))
			throw new Exception("unsupported type: "+tag.getChildValue("type"));
		if(!tag.getChildValue("protocol_version").equals("1.0"))
			throw new Exception("unsupported protocol_version: "+tag.getChildValue("protocol_version"));
		String nickname = tag.getChildValue("nickname");
		int rc_id = Integer.parseInt(tag.getChildValue("rc_id"));
		int vt_id = Integer.parseInt(tag.getChildValue("vt_id"));

		RCRSVirtualSoccerEcoBe ecobe = findEcoBe(rc_id);
//		Mark mark = findMark(vt_id);
		
		ecobe.setClient(address);
		ecobe.update(packet);



		send(ecobe.createPacket("<connect>true</connect>"));
		send(ecobe.createPacket("<connect><type>Client</type><protocol_version>1.0</protocol_version><movement_mode>wheel_velocities</movement_mode></connect>"));
		
		
		
		
//		for(int i=0;i<dslistener_list_.size();i++)
//			dslistener_list_.get(i).failedToConnect(address, "???");

		log("Connected new client: "+ecobe);
	}
*/
	public RCRSVirtualSoccerEcoBe findEcoBe(InetSocketAddress address) {
		for(RCRSVirtualSoccerEcoBe ecobe : ecobe_list_) {
//System.out.println("available address: "+address);
			if(address.equals(ecobe.getRemoteAddress())) {
//				System.out.println("find address: "+address);
				return ecobe;
			}
		}
		System.out.println("cannnot find address: "+address);
		return null;
	}
	public RCRSVirtualSoccerEcoBe findEcoBe(int id) throws Exception {
		for(int i=0;i<ecobe_list_.size();i++)
			if(ecobe_list_.get(i).getID() == id)
				return ecobe_list_.get(i);
		throw new Exception("cannot find EcoBe: "+id);
	}
	public Mark findMark(int id) throws Exception {
		for(int i=0;i<mark_list_.size();i++)
			if(mark_list_.get(i).getID() == id)
				return mark_list_.get(i);
		throw new Exception("cannot find Mark: "+id);
	}

/*
	public void start() throws Exception {

		init();

		String message = null;
		TagElement response = null;
		try{
			message = receive();
			response = createResponseMessage(true);
			
			send(response);
			
			send("<connect><type>Client</type><protocol_version>1.0</protocol_version><movement_mode>wheel_velocities</movement_mode></connect>");
			message = receive();
//
String sampledata = "<WorldData><time>5.25131</time><agent_id>0</agent_id><nickname>player1</nickname><status>found</status><max_agent>2</max_agent><playMode>play on</playMode><score><yellow>0</yellow><blue>0</blue></score><wheel_velocities><right>0</right><left>0</left></wheel_velocities><ball><dist>117</dist><angle>39</angle></ball><teamMate><id>1</id><nickname>player2</nickname><status>found</status><dist>89</dist><angle>48</angle><orientation>107</orientation></teamMate><teamMate><id>2</id><nickname>player3</nickname><status>found</status><dist>79</dist><angle>-21</angle><orientation>103</orientation></teamMate><teamMate><id>3</id><nickname>player4</nickname><status>found</status><dist>74</dist><angle>-57</angle><orientation>273</orientation></teamMate><teamMate><id>4</id><nickname>player0</nickname><status>found</status><dist>70</dist><angle>66</angle><orientation>258</orientation></teamMate><opponent><id>5</id><nickname>player7</nickname><status>found</status><dist>167</dist><angle>46</angle><orientation>161</orientation></opponent><opponent><id>6</id><nickname>player9</nickname><status>found</status><dist>56</dist><angle>3</angle><orientation>249</orientation></opponent><opponent><id>7</id><nickname>player5</nickname><status>found</status><dist>287</dist><angle>58</angle><orientation>357</orientation></opponent><opponent><id>8</id><nickname>player8</nickname><status>found</status><dist>53</dist><angle>0</angle><orientation>328</orientation></opponent><opponent><id>9</id><nickname>player6</nickname><status>found</status><dist>128</dist><angle>-34</angle><orientation>6</orientation></opponent><flag><id>top_left_pole</id><dist>152</dist><angle>-59</angle></flag><flag><id>bottom_left_pole</id><dist>116</dist><angle>-83</angle></flag><flag><id>top_right_pole</id><dist>310</dist><angle>57</angle></flag><flag><id>bottom_right_pole</id><dist>294</dist><angle>69</angle></flag><flag><id>top_left_corner</id><dist>236</dist><angle>-40</angle></flag><flag><id>bottom_left_corner</id><dist>126</dist><angle>-133</angle></flag><flag><id>top_right_corner</id><dist>359</dist><angle>41</angle></flag><flag><id>bottom_right_corner</id><dist>298</dist><angle>89</angle></flag><flag><id>top_left_goal</id><dist>148</dist><angle>-8</angle></flag><flag><id>bottom_left_goal</id><dist>14</dist><angle>85</angle></flag><flag><id>top_right_goal</id><dist>224</dist><angle>36</angle></flag><flag><id>bottom_right_goal</id><dist>169</dist><angle>78</angle></flag><flag><id>top_left_small_area</id><dist>83</dist><angle>-74</angle></flag><flag><id>bottom_left_small_area</id><dist>128</dist><angle>-48</angle></flag><flag><id>top_right_small_area</id><dist>259</dist><angle>68</angle></flag><flag><id>bottom_right_small_area</id><dist>277</dist><angle>55</angle></flag><flag><id>middle_center</id><dist>117</dist><angle>39</angle></flag><flag><id>top_center</id><dist>229</dist><angle>11</angle></flag><flag><id>bottom_center</id><dist>112</dist><angle>112</angle></flag></WorldData>";

			for(int i=0;i<10;i++) {
				send(sampledata);
				message = receive();
//				System.out.println("received command:"+message);
			}
		}catch(Exception e){
			response = createResponseMessage(false);
		}
		
//alert("Dummy Server received message: \n"+message);
	}
*/

	public void init() throws Exception {
		initEcoBe();
		initSocket();
	}
	
	public void initEcoBe() throws Exception {
		ecobe_list_.clear();
		mark_list_.clear();
		yellow_team_member_list_.clear();
		blue_team_member_list_.clear();
		
		TagElement[] ecobe_tag = config_.getTag("team_yellow/ecobe_list").getTagChildren("ecobe");
		for(int i=0;i<ecobe_tag.length;i++) {
			int vt_id = Integer.parseInt(ecobe_tag[i].getChildValue("vt_id"));
			int rc_id = Integer.parseInt(ecobe_tag[i].getChildValue("rc_id"));
			Mark mark = new Mark(vt_id);
			RCRSVirtualSoccerEcoBe ecobe = new RCRSVirtualSoccerEcoBe(rc_id);
			ecobe.setTeamName("yellow");
			ecobe.putMark(mark);
			mark_list_.add(mark);
			ecobe_list_.add(ecobe);
			yellow_team_member_list_.add(ecobe);
		}

		ecobe_tag = config_.getTag("team_blue/ecobe_list").getTagChildren("ecobe");
		for(int i=0;i<ecobe_tag.length;i++) {
			int vt_id = Integer.parseInt(ecobe_tag[i].getChildValue("vt_id"));
			int rc_id = Integer.parseInt(ecobe_tag[i].getChildValue("rc_id"));
			Mark mark = new Mark(vt_id);
			RCRSVirtualSoccerEcoBe ecobe = new RCRSVirtualSoccerEcoBe(rc_id);
			ecobe.setTeamName("blue");
			ecobe.putMark(mark);
			mark_list_.add(mark);
			ecobe_list_.add(ecobe);
			blue_team_member_list_.add(ecobe);
		}
//		javax.swing.JOptionPane.showMessageDialog(null, yellow_team_member_list_.size()+","+blue_team_member_list_.size());
	}
	
	private void log(String message) {
		for(int i=0;i<dslistener_list_.size();i++)
			dslistener_list_.get(i).log(message);
	}
	

	public TagElement createFirstMessage() throws Exception {
		TagElement message = new TagElement("connect");
		message.setChildren(
			new TagElement("type", config_.getValue("connection/type", "Client")),
			new TagElement("protocol_version", config_.getValue("connection/protocol_version", "1.0")),
			new TagElement("nickname", config_.getValue("connection/nickname", "no_name")),
			new TagElement("rc_id", config_.getValue("connection/rc_id", "0")),
			new TagElement("vt_id", config_.getValue("connection/vt_id", "0"))
		);
		return message;
	}

	public void initSocket() throws Exception {
		int timeout = config_.getInt("protocol/timeout", 1000);
		int message_max_length = config_.getInt("protocol/message_max_length", 1024*4);
		int blue_port = config_.getInt("team_blue/port", 3310);
		int yellow_port = config_.getInt("yellow_blue/port", 3311);
		System.out.println("Server port blue: "+blue_port);
		System.out.println("Server port yellow: "+yellow_port);
		System.out.println("message max length: "+message_max_length);
		System.out.println("timeout: "+timeout);

		blue_packet_ = new DatagramPacket(new byte[0], 0);
		blue_socket_ = new DatagramSocket(blue_port);
		blue_socket_.setSoTimeout(timeout);
		blue_receive_buf_ = new byte[message_max_length];

		yellow_packet_ = new DatagramPacket(new byte[0], 0);
		yellow_socket_ = new DatagramSocket(yellow_port);
		yellow_socket_.setSoTimeout(timeout);
		yellow_receive_buf_ = new byte[message_max_length];
/*
		int blue_port = config_.getInt("server/port", 3310);
		System.out.println("Server port: "+blue_port);
		System.out.println("timeout: "+timeout);
		blue_packet_ = new DatagramPacket(new byte[0], 0);
		blue_socket_ = new DatagramSocket(port);
		blue_socket_.setSoTimeout(timeout);
		blue_receive_buf_ = new byte[config_.getInt("server/message_max_length", 1024*4)];
*/
	}
	
	public void send(TagElement tag, DatagramPacket packet_, DatagramSocket socket_) throws Exception {
		send(tag.toString(), packet_, socket_);
	}
	public void send(String message, DatagramPacket packet_, DatagramSocket socket_) throws Exception {
		byte[] send_buf = message.getBytes();
		packet_.setData(send_buf);
		socket_.send(blue_packet_);
		System.out.println("send:" + message);
	}
/*
	public void send(String message) throws Exception {
		byte[] send_buf = message.getBytes();
		blue_packet_.setData(send_buf);
		blue_socket_.send(blue_packet_);
		System.out.println("send:" + message);
	}
*/
/*
	public void send(DatagramPacket packet) throws Exception {
		blue_socket_.send(packet);
	}
*/
/*
	public void send(String message, DatagramPacket received_packet, DatagramSocket socket_) throws Exception {
		byte[] send_buf = message.getBytes();
		received_packet.setData(send_buf);
		socket_.send(received_packet);
		System.out.println("send:" + message);
	}
*/
	public TagElement createResponseMessage(boolean success) throws Exception {
		TagElement message = new TagElement("connect", (success ? "true" : "false"));
		return message;
	}

	public String receive() throws Exception {
//		System.out.println("wait receiving...");
		blue_packet_.setData(blue_receive_buf_);
		blue_socket_.receive(blue_packet_);
		
		String response = new String(blue_packet_.getData(), 0, blue_packet_.getLength());
		System.out.println("receive:" + response);
		return response;
	}
	
	public String createMessage(RCRSVirtualSoccerEcoBe ecobe) {
		int member_length = 4;  // member_length = {1(me)+teammate_length} + {opponent_length}
		int opponent_length = member_length/2;
		int teammate_length = opponent_length-1;
		
		StringBuffer sb = new StringBuffer();
		sb.append("<WorldData>");
		sb.append(createKeyValueTag("time", String.valueOf(time_)));
		sb.append(createKeyValueTag("agent_id", ecobe.getID()));
		sb.append(createKeyValueTag("nickname", "player1"));
		sb.append(createKeyValueTag("status", "found"));
		sb.append(createKeyValueTag("max_agent", ecobe_list_.size()));
		sb.append(createKeyValueTag("playMode", "play on"));
		sb.append("<score>");
		sb.append(createKeyValueTag("yellow", "0"));
		sb.append(createKeyValueTag("blue", "0"));
		sb.append("</score>");
		sb.append("<wheel_velocities>");
		sb.append(createKeyValueTag("right", "0"));
		sb.append(createKeyValueTag("left", "0"));
		sb.append("</wheel_velocities>");
		sb.append(createLocationTag("ball", ecobe, ball_.getLocation().getX(), ball_.getLocation().getY()));


/*
		int now_teammate_count = 0;
		for(;now_teammate_count<blue_team_member_list_.size();now_teammate_count++) {
			if(blue_team_member_list_.get(now_teammate_count)==ecobe)
			sb.append(createMemberTag("teamMate", blue_team_member_list_.get(now_teammate_count)));
		}
		for(int i=0;i<yellow_team_member_list_.size();i++)
			sb.append(createMemberTag("opponent", yellow_team_member_list_.get(i)));

*/
		ArrayList<RCRSVirtualSoccerEcoBe> opponent_list, teammate_list;
		if(ecobe.getTeamName().equals("blue")) {
			teammate_list = blue_team_member_list_;
			opponent_list = yellow_team_member_list_;
		} else {
			opponent_list = blue_team_member_list_;
			teammate_list = yellow_team_member_list_;
		}
		
		for(int i=0; i<teammate_list.size(); i++) {
			RCRSVirtualSoccerEcoBe target = teammate_list.get(i);
			if(target != ecobe) {
				sb.append(createMemberTag("teamMate", target, ecobe));
			}
		}
		for(int i=0; i<opponent_list.size(); i++) {
			RCRSVirtualSoccerEcoBe target = opponent_list.get(i);
			sb.append(createMemberTag("opponent", target, ecobe));
		}


		for(Iterator<String> it = flag_location_map_.keySet().iterator(); it.hasNext(); ) {
			String key = it.next();
			sb.append(createFlagTag(key, ecobe, flag_location_map_.get(key)));
		}

		sb.append("</WorldData>");
		return sb.toString();
	}

	public String createLocationTag(String key, EcoBe ecobe, double x, double y) {
		double rx = x - ecobe.getLocation().getX();
		double ry = y - ecobe.getLocation().getY();
		double ecobe_angle = ecobe.getAngle();
		int angle = (int)((Math.atan2(ry, rx)-ecobe_angle)*180.0/Math.PI);
		int dist = (int)(Math.sqrt(rx*rx + ry*ry));
		return "<"+key+">"
			+ "<dist>"+dist+"</dist>"
			+ "<angle>"+angle+"</angle>"
			+ "</"+key+">";

	}
	public String createFlagTag(String key, EcoBe ecobe, Point2D.Double p) {
//		EcoBe ecobe = client.getEcoBe();
		double x = p.getX() - ecobe.getLocation().getX();
		double y = p.getY() - ecobe.getLocation().getY();
		double ecobe_angle = ecobe.getAngle();
		int angle = (int)((Math.atan2(y, x)-ecobe_angle)*180/Math.PI);
		int dist = (int)(Math.sqrt(x*x + y*y));
		return "<flag>"
		+ createKeyValueTag("id", key)
		+ createKeyValueTag("dist", dist)
		+ createKeyValueTag("angle", angle)
		+ "</flag>";
	}
	public String createKeyValueTag(String key, double	 value) {
		return createKeyValueTag(key, String.valueOf(value));
	}
	public String createKeyValueTag(String key, int value) {
		return createKeyValueTag(key, String.valueOf(value));
	}
	public String createKeyValueTag(String key, String value) {
		return "<"+key+">"+value+"</"+key+">";
	}
	
	public String createMemberTag(String team, RCRSVirtualSoccerEcoBe ecobe, RCRSVirtualSoccerEcoBe me) {
		double x = ecobe.getLocation().getX() - me.getLocation().getX();
		double y = ecobe.getLocation().getY() - me.getLocation().getY();
		double ecobe_angle = ecobe.getAngle();
		double me_angle = me.getAngle();
		int dist = (int)(Math.sqrt(x*x + y*y));
		int orientation = (int)(ecobe_angle-me_angle);
		int angle = (int)((Math.atan2(y, x)-me_angle)*180.0/Math.PI);
//System.out.println("member tag: angle:"+angle+"; dist:"+dist);
		return "<"+team+">"
		+ createKeyValueTag("id", ecobe.getID())
		+ createKeyValueTag("nickname", ecobe.getNickName())
		+ createKeyValueTag("status", "found")
		+ createKeyValueTag("dist", dist)
		+ createKeyValueTag("angle", angle)
		+ createKeyValueTag("orientation", orientation)
		+"</"+team+">";
	}
/*
	public String createMemberTag(String team, SoccerClient client) {
		EcoBe ecobe = client.getEcoBe();
		return "<"+team+">"
		+ createKeyValueTag("id", ecobe.getID())
		+ createKeyValueTag("nickname", ecobe.getNickName())
		+ createKeyValueTag("status", "found")
		+ createKeyValueTag("dist", (int)(Math.random()*100))
		+ createKeyValueTag("angle", (int)(Math.random()*100))
		+ createKeyValueTag("orientation", (int)(Math.random()*100))
		+"</"+team+">";
	}
*/
	public static void alert(Object message) {
		javax.swing.JOptionPane.showMessageDialog(null, message);
	}

}
