package org.server;

import java.util.*;
import java.io.*;
import java.net.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import org.server.motion.*;

public class DummyServerGUI extends JPanel {

	private JPanel state_panel_;
	private JPanel player_panel_;
	private JTextArea log_area_;
	private DummyServer server_;
	private SoccerField field_;
	private JComboBox motion_simulator_list_;

	public DummyServerGUI(DummyServer server) {
		server_ = server;
		player_panel_ = new JPanel();
		player_panel_.setPreferredSize(new Dimension(200, 200));
		player_panel_.setLayout(new BoxLayout(player_panel_, BoxLayout.Y_AXIS));

		motion_simulator_list_ = new JComboBox(new String[]{"None", "org.server.motion.SimpleMotionSimulator"});
		motion_simulator_list_.setEditable(true);
		motion_simulator_list_.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				JComboBox c = (JComboBox)e.getSource();
				String item = (String)c.getSelectedItem();
				try{
					if("None".equals(item)) {
						server_.setMotionSimulator(null);
					} else {
						server_.setMotionSimulator((MotionSimulator)(Class.forName(item).newInstance()));
					}
				}catch(Exception exc){
					exc.printStackTrace();
				}
			}
		});
		
		JPanel control_panel_ = new JPanel();
		control_panel_.setLayout(new BoxLayout(control_panel_, BoxLayout.Y_AXIS));
		control_panel_.add(motion_simulator_list_);

		state_panel_ = new JPanel();
		state_panel_.setLayout(new BoxLayout(state_panel_, BoxLayout.Y_AXIS));
		state_panel_.add(player_panel_);
		
		field_ = new SoccerField(server);
		field_.createAndShowGUI();
		
		server_.addDummyServerListener(new DummyServerListener(){
			public void connectionDetected(InetSocketAddress address) {
				player_panel_.add(new JLabel(address.toString()));
				player_panel_.revalidate();
				alert(address);
			}
			public void failedToConnect(InetSocketAddress address, String message) {
				alert(address+","+message);
			}
			public void update() {
				field_.update();
			}
			public void log(String message) {
				log_area_.append(message+"\n");
				log_area_.setCaretPosition(log_area_.getDocument().getLength());
			}
		});
		log_area_ = new JTextArea();
		setLayout(new BorderLayout());
		add(control_panel_, BorderLayout.NORTH);
		add(state_panel_, BorderLayout.CENTER);
		JScrollPane sp = new JScrollPane(log_area_);
		sp.setPreferredSize(new Dimension(100, 100));
		add(sp, BorderLayout.SOUTH);
		
		JFrame frame = new JFrame("Dummy Server");
		frame.setContentPane(this);
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.setSize(500, 500);
		frame.setVisible(true);
	}
	
	public static void alert(Object message) {
		System.out.println(message);
		// javax.swing.JOptionPane.showMessageDialog(null, message);
	}
}
