package org.server.motion;

import java.util.*;
import java.io.*;
import java.net.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.awt.geom.*;
import org.realobject.*;

public class SimpleMotionSimulator implements MotionSimulator {

	private HashMap<String, Point2D.Double> flag_location_map_;
	private ArrayList<RCRSVirtualSoccerEcoBe> ecobe_list_;
	private Ball ball_;
	
	public void init(HashMap<String, Point2D.Double> flag_location_map, ArrayList<RCRSVirtualSoccerEcoBe> ecobe_list, Ball ball) {
		flag_location_map_ = flag_location_map;
		ecobe_list_ = ecobe_list;
		ball_ = ball;
	}
	
	public void simulateStep(double step_time) {
		ball_.setForce(0, 0);
		Point2D.Double v = ball_.getVelocity();
		ball_.setVelocity(v.getX()*0.9, v.getY()*0.9);
		
		for(int i=0; i<ecobe_list_.size(); i++) {
			RCRSVirtualSoccerEcoBe target = ecobe_list_.get(i);
			String command_type = target.getCommandType();
			double[] command_args = target.getCommandArgs();
			if(command_type==null) continue;
			if(command_type.equals("kick")) {
				double distance = ball_.getLocation().distance(target.getLocation());
				if(distance <= (ball_.getRadius()+target.getRadius())) {
					double ecobe_angle = target.getAngle();
					double angle = command_args[0]*Math.PI/180.0+ecobe_angle;
					double force = 10*command_args[1];
					ball_.addForce(force*Math.cos(angle), force*Math.sin(angle));
				}
			}else if(command_type.equals("wheel_velocities")) {
				double left = command_args[0];
				double right = command_args[1];
				double drive = 0.1*(left+right)/2;
				double rotate = (right-left)/2;
				double angle = target.getAngle();
				target.move(drive*Math.cos(angle), drive*Math.sin(angle));
				target.rotate(rotate);
			}
		}
		Point2D.Double f = ball_.getForce();
		Point2D.Double a = new Point2D.Double(f.getX()/ball_.getMass(), f.getY()/ball_.getMass());
		v = ball_.getVelocity();
		ball_.setVelocity(v.getX()+a.getX()*step_time, v.getY()+a.getY()*step_time);
		ball_.move(v.getX()*step_time, v.getY()*step_time);
	}
}
