package org.protocol;

import java.util.*;
import java.io.*;
import java.net.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;

public class SimpleUDPSocket {

	private InputStream in_;
	private OutputStream out_;
	private DatagramPacket packet_;
	private DatagramSocket socket_;
	private String ip_;
	private int port_;
	private int timeout_;
	private int buf_size_;
	private boolean is_server_mode_;

	public SimpleUDPSocket(String ip, int port, int timeout, int buf_size) throws Exception {
		is_server_mode_ = true;
		ip_ = ip;
		port_ = port;
		timeout_ = timeout;
		buf_size_ = buf_size;
		init();
	}

	public void init() throws Exception {
		byte[] buf_ = new byte[buf_size_];
		InetSocketAddress address = new InetSocketAddress(ip_, port_);
		packet_ = new DatagramPacket(buf_, buf_.length, address);
		socket_ = new DatagramSocket();

		in_ = new InputStream() {
			int c = 'a';
			byte[] receive_buf = new byte[buf_size_];
			int length = 0;
			int offset = 0;
			public int read () {
				if(offset==length) {
					receive();
					if(offset==0)
						return -1;
				}
				return receive_buf[offset++];
			}
			public void receive() {
				try{
					packet_.setData(receive_buf, 0, receive_buf.length);
					socket_.receive(packet_);
					receive_buf = packet_.getData();
					offset = 0;
					length = packet_.getLength();
				}catch(Exception e){
					e.printStackTrace();
				}
			}
		};

		out_ = new OutputStream() {
			byte[] send_buf = new byte[buf_size_];
			int length = 0;
			int offset = 0;
			public void write(int c) {
				send_buf[length++] = (byte)c;
//				System.out.println("write: "+(char)c);
			}
			public void flush() {
				try{
					packet_.setData(send_buf, offset, length);
					socket_.send(packet_);
					offset = 0;
					length = 0;
				}catch(Exception e){
					e.printStackTrace();
				}
			}
		};
	}
	public InputStream getInputStream() {
		return in_;
	}
	public OutputStream getOutputStream() {
		return out_;
	}

	public static void main(String[] args) throws Exception	 {
		SimpleUDPSocket socket = new SimpleUDPSocket("localhost", 3310, 0, 1024);
		InputStream in = socket.getInputStream();
		OutputStream out = socket.getOutputStream();
		for(int i=0;i<1000;i++) {
			byte[] buf = (String.valueOf(i)+"aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa").getBytes();
			out.write(buf);
			out.flush();
//			Thread.sleep(5);
		}
		
		for(int i=in.read(); i!=-1; i=in.read())
			System.out.println("receive: " + (char)i);
		in.close();
		out.close();
	}
	public static void main2(String[] args) throws Exception	 {
		SimpleUDPSocket socket = new SimpleUDPSocket("localhost", 3310, 0, 1024);
		InputStream in = socket.getInputStream();
		OutputStream out = socket.getOutputStream();
		byte[] buf = "<connect><type>Client</type><protocol_version>1.0</protocol_version><nickname>test</nickname><rc_id>0</rc_id><vt_id>0</vt_id></connect>".getBytes();
		out.write(buf);
		out.flush();
		Thread.sleep(1000);
		out.write(buf);
		out.flush();
		
		for(int i=in.read(); i!=-1; i=in.read())
			System.out.println("receive: " + (char)i);
		in.close();
		out.close();
	}

	public static void alert(Object message) {
		System.out.println(message);
		// javax.swing.JOptionPane.showMessageDialog(null, message);
	}
}
